<?php

namespace Drupal\pagedesigner_block\Plugin\pagedesigner\Handler;

use Drupal\block\Entity\Block as BlockEntity;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Render\Markup;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\Plugin\HandlerPluginBase;
use Drupal\ui_patterns\Definition\PatternDefinition;

/**
 * Process entities of type "block".
 *
 * @PagedesignerHandler(
 *   id = "block",
 *   name = @Translation("Block handler"),
 *   types = {
 *      "block"
 *   },
 * )
 */
class Block extends HandlerPluginBase {

  /**
   * The definition of the block pattern.
   *
   * @var array
   */
  protected $definition = [
    "id" => 'block:',
    "pagedesigner" => 1,
    "icon" => "fab fa-drupal",
    "type" => "block",
    "category" => 'Drupal blocks',
    "label" => '',
    "description" => '',
    "autorender" => FALSE,
  ];

  /**
   * {@inheritdoc}
   */
  public function collectAttachments(array &$attachments) {
    $attachments['library'][] = 'pagedesigner_block/pagedesigner';
  }

  /**
   * {@inheritdoc}
   */
  public function collectPatterns(array &$patterns) {
    $blockIds = \Drupal::entityQuery('block')
      ->condition('region', 'pagedesigner')
      ->condition(
        'theme',
        \Drupal::service('theme.manager')
          ->getActiveTheme()
          ->getName()
        )
      ->execute();
    foreach ($blockIds as $blockId) {
      $definition = $this->definition;
      $id = $definition['id'] . $blockId;
      $cid = 'pagedesigner_definition_block_' . $blockId;
      if ($item = \Drupal::cache()->get($cid)) {
        $patterns[$id] = new PatternDefinition($item->data);
      }
      else {
        $block = BlockEntity::load($blockId);
        if ($block != NULL) {
          $definition['block'] = $blockId;
          $build = \Drupal::entityTypeManager()
            ->getViewBuilder('block')
            ->view($block);
          $markup = \Drupal::service('renderer')->renderPlain($build);
          $markup = '<div data-gjs-type="block">' . $markup . '<div>';
          $definition['markup'] = $markup;
          $definition['label'] = (string) $block->label();
          $patterns[$id] = new PatternDefinition($definition);
          \Drupal::cache()->set($cid, $definition, Cache::PERMANENT, $block->getCacheTags());
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function get(Element $entity, string &$result = '') {
    // TODO: Find a minimal representation of a block for search.
    $result = $entity->field_block->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function serialize(Element $entity, array &$result = []) {
    $result['block_id'] = $entity->field_block->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function render(Element $entity, array &$build = []) {
    if ($entity->field_block->entity == NULL) {
      return;
    }
    $build = \Drupal::entityTypeManager()
      ->getViewBuilder('block')
      ->view($entity->field_block->entity) + $build;
  }

  /**
   * {@inheritdoc}
   */
  public function renderForEdit(Element $entity, array &$build = []) {
    if ($entity->field_block->entity == NULL) {
      return [];
    }
    $show = TRUE;
    foreach ($entity->field_block->entity->getVisibilityConditions() as $condition_id => $condition) {
      if ($condition_id == 'pagedesigner_condition') {
        $show = $condition->execute();
      }
    }
    if (!$show) {
      $markup = '<pre style="text-align:center">Placeholder for block ' . $entity->field_block->entity->label() . '</pre>';
    }
    else {
      $subBuild = $this->elementHandler->render($entity);
      $markup = \Drupal::service('renderer')->render($subBuild);
    }
    $build = [
      '#type' => 'inline_template',
      '#template' => '<div data-gjs-type="block:' . $entity->field_block->target_id . '" data-entity-id="{{id}}" id="{{html_id}}">{{markup}}</div>',
      '#context' => [
        'markup' => [
          '#markup' => Markup::create($markup),
        ],
        'id' => $entity->id(),
        'html_id' => 'pd-cp-' . $entity->id(),
      ],
    ] + $build;
  }

  /**
   * {@inheritdoc}
   */
  public function describe(Element $entity, array &$result = []) {
    parent::describe($entity, $result);
    $result['type'] = 'block';
  }

  /**
   * {@inheritdoc}
   */
  public function generate($definition, array $data, Element &$entity = NULL) {
    $blockId = $definition['additional']['block'];
    $block = BlockEntity::load($blockId);
    if ($block == NULL) {
      throw new BadRequestHttpException('The given block does not exist.');
    }

    parent::generate(['type' => 'block', 'name' => 'block'], $data, $entity);
    $entity->field_block->entity = $block;
    $entity->parent->target_id = $data['parent'];
    $entity->container->target_id = $data['container'];
    $entity->entity->target_id = $data['entity'];
    $entity->saveEdit();

    // Adding entity to parent cell if provided.
    $cell = $entity->parent->entity;
    if ($cell != NULL) {
      $cell = $cell->getTranslation($entity->langcode->value);
      $cell->children->appendItem($entity);
      $cell->saveEdit();
    }
  }

}
