<?php

namespace Drupal\pagedesigner\Plugin;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManager;
use Drupal\Core\Session\AccountInterface;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\Plugin\pagedesigner\HandlerPluginInterface;
use Drupal\pagedesigner\Service\ElementHandler;
use Drupal\ui_patterns\Definition\PatternDefinitionField;
use Drupal\ui_patterns\UiPatternsManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;

/**
 * Base implementation for pagedesigner element handlers.
 */
abstract class HandlerPluginBase extends PluginBase implements HandlerPluginInterface, PluginInspectionInterface, ContainerFactoryPluginInterface {

  /**
   * The user to render the markup for.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $user = NULL;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManager
   */
  protected $languageManager = NULL;

  /**
   * The pattern manager.
   *
   * @var \Drupal\ui_patterns\UiPatternsManager
   */
  protected $patternManager = NULL;

  /**
   * The element handler.
   *
   * @var \Drupal\pagedesigner\Service\ElementHandler
   */
  protected $elementHandler = NULL;

  /**
   * The pattern definitions.
   *
   * @var array
   */
  protected static $patternDefinitions = NULL;

  /**
   * Create a new instance.
   */
  public function __construct($configuration,
        $plugin_id,
        $plugin_definition,
        AccountInterface $user,
        LanguageManager $language_manager,
        UiPatternsManager $pattern_manager,
        ElementHandler $element_handler) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->user = $user;
    $this->languageManager = $language_manager;
    $this->patternManager = $pattern_manager;
    $this->elementHandler = $element_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
          $configuration,
          $plugin_id,
          $plugin_definition,
          $container->get('current_user'),
          $container->get('language_manager'),
          $container->get('plugin.manager.ui_patterns'),
          $container->get('pagedesigner.service.element_handler')
      );
  }

  /**
   * {@inheritdoc}
   */
  public function collectAttachments(array &$attachments) {

  }

  /**
   * {@inheritdoc}
   */
  public function collectPatterns(array &$patterns) {

  }

  /**
   * {@inheritdoc}
   */
  public function adaptPatterns(array &$patterns) {

  }

  /**
   * {@inheritdoc}
   */
  public function prepare(PatternDefinitionField &$field, array &$fieldArray) {

  }

  /**
   * {@inheritdoc}
   */
  public function get(Element $entity, string &$result = '') {
    if ($entity->hasField('field_content')) {
      $result .= $entity->field_content->value;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getContent(Element $entity, array &$list = []) {
    $entity = $entity->loadNewestPublished();
    if ($entity != NULL) {
      $value = $this->elementHandler->get($entity);
      $list[] = $value;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function patch(Element $entity, array $data) {
    if (isset($data[0]) && \is_scalar($data[0])) {
      $entity->field_content->value = $data[0];
      $entity->saveEdit();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function generate($definition, array $data, Element &$entity = NULL) {
    $type = $definition['type'];
    $name = (!empty($definition['name'])) ? $definition['name'] : $definition['type'];
    $language = $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT);
    $entity = Element::create(
      [
        'type' => $type,
        'name' => $name,
        'langcode' => $language,
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function copy(Element $entity, Element $container = NULL, Element &$clone = NULL) {
    $clone = $entity->createDuplicate();
    $clone->container->entity = $container;
    $clone->langcode->value = $entity->langcode->value;
    if ($container != NULL) {
      $clone->entity->entity = $container->entity->entity;
      $clone->langcode->value = $container->langcode->value;
    }
    else {
      $clone->entity->entity = NULL;
    }

    $children = [];
    foreach ($entity->children as $item) {
      if ($item->entity != NULL) {
        $child = $this->elementHandler->copy($item->entity, $container);
        $child->langcode->value = $clone->langcode->value;
        $child->container->entity = $container;
        if ($container != NULL) {
          $child->entity->entity = $container->entity->entity;
        }
        else {
          $child->entity->entity = NULL;
        }
        $child->parent->entity = $clone;
        $child->setPublished(FALSE);
        $child->save();
        $children[] = $child->id();
      }
    }
    $clone->children->setValue($children);
    $clone->setPublished(FALSE);
    $clone->save();
  }

  /**
   * {@inheritdoc}
   */
  public function delete(Element $entity, bool $remove = FALSE) {
    foreach ($entity->children as $item) {
      if ($item->entity != NULL) {
        $this->elementHandler->delete($item->entity, $remove);
      }
    }
    if ($remove) {
      $entity->delete();
    }
    else {
      $entity->deleted = TRUE;
      $entity->saveEdit();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function restore(Element $entity) {
    foreach ($entity->children as $item) {
      if ($item->entity != NULL) {
        $this->elementHandler->restore($item->entity);
      }
    }
    $entity->deleted = FALSE;
    $entity->save();
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->pluginDefinition['name'];
  }

  /**
   * {@inheritdoc}
   */
  public function renderForPublic(Element $entity, array &$build) {
    $entity = $entity->loadNewestPublished();
    if ($entity != NULL) {
      $this->elementHandler->render($entity, $build) + $build;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function render(Element $entity, array &$build = []) {
    $value = $this->elementHandler->get($entity);
    $build = ['#markup' => $value, '#pagedesigner_entity' => $entity] + $build;
    \Drupal::service('renderer')->addCacheableDependency($build, $entity);
  }

  /**
   * {@inheritdoc}
   */
  public function renderForEdit(Element $entity, array &$build = []) {
    $build = $this->elementHandler->render($entity, $build) + $build;
    $build['#pagedesigner_edit'] = TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function serialize(Element $entity, array &$result = []) {
    $result = ['entity' => $entity] + $result;
  }

  /**
   * {@inheritdoc}
   */
  public function describe(Element $entity, array &$result = []) {
    $result['id'] = $entity->id();
    $result['parent'] = $entity->parent->target_id;
    $result['container'] = $entity->container->target_id;
    $result['entity'] = $entity->entity->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function publish(Element $entity) {
    foreach ($entity->children as $item) {
      $this->elementHandler->publish($item->entity);
    }
    if (!$entity->isPublished()) {
      $entity->setPublished(TRUE);
      $entity->save();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function unpublish(Element $entity) {
    foreach ($entity->children as $item) {
      $this->elementHandler->unpublish($item->entity);
    }
    if ($entity->isPublished()) {
      $entity->setPublished(FALSE);
      $entity->save();
    }
  }

  /**
   * Returns the definition of the specified pattern.
   *
   * @param string $id
   *   The id of the pattern.
   *
   * @return \Drupal\ui_patterns\Definition\PatternDefinition|null
   *   The pattern or null.
   */
  protected function getPatternDefinition($id) {
    if (self::$patternDefinitions == NULL) {
      self::$patternDefinitions = $this->patternManager->getDefinitions();
      $this->elementHandler->collectPatterns(self::$patternDefinitions);
    }
    if (!empty(self::$patternDefinitions[$id])) {
      return self::$patternDefinitions[$id];
    }
    return NULL;
  }

}
