<?php

namespace Drupal\pagedesigner\Service;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Render\Markup;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\PagedesignerService;
use Drupal\pagedesigner\Service\Render\Styles;
use Drupal\user\Entity\User;

/**
 * The renderer service allows to render a pagedesigner element container.
 */
class Renderer extends PagedesignerService {

  /**
   * Style object to collect the styles during rendering.
   *
   * @var Drupal\pagedesigner\Service\Render\Styles|null
   */
  protected static $styles = NULL;

  /**
   * Whether rendering is in process.
   *
   * @var bool
   */
  protected $rendering = FALSE;

  /**
   * Add a style definition.
   *
   * @param string $key
   *   The size for which to add the style definition (mn, sm, xs).
   * @param string $style
   *   The style definition to add.
   * @param int $id
   *   The id of the element.
   */
  public static function addStyle($key, $style, $id) {
    if (self::$styles == NULL) {
      self::$styles = new Styles();
    }
    self::$styles->addStyle($key, $style, $id);
  }

  /**
   * Publicly render a container for a certain entity.
   *
   * @param \Drupal\pagedesigner\Entity\Element $container
   *   The container to render.
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity the container is rendered for.
   *
   * @return \Drupal\pagedesigner\Service\Renderer
   *   The renderer service.
   */
  public function renderForPublic(Element $container, ContentEntityBase $entity = NULL) {
    if ($container == NULL) {
      return $this;
    }
    if (empty($this->_output)) {
      $this->_output = [];
    }
    if (!$this->rendering) {
      self::$styles = NULL;
    }
    $this->rendering = TRUE;
    $this->_output = $this->getHandler($container)->renderForPublic($container);
    $this->rendering = FALSE;
    $this->addStyles($container);
    $this->addCacheInformation($container, $entity, 'public');
    // $this->addPatternAttachments();
    return $this;
  }

  /**
   * Render a container for a certain entity.
   *
   * @param \Drupal\pagedesigner\Entity\Element $container
   *   The container to render.
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity the container is rendered for.
   *
   * @return \Drupal\pagedesigner\Service\Renderer
   *   The renderer service.
   */
  public function render(Element $container, ContentEntityBase $entity = NULL, $nocache = FALSE) {
    if ($container == NULL) {
      return $this;
    }
    if (empty($this->_output)) {
      $this->_output = [];
    }
    if (!$this->rendering) {
      self::$styles = NULL;
    }
    $this->rendering = TRUE;
    $this->_output = $this->getHandler($container)->render($container);
    $this->rendering = FALSE;
    if ($nocache) {
      $this->_output['#cache']['max-age'] = 0;
    }
    if (empty($this->_output['#attached'])) {
      $this->_output['#attached'] = [];
    }
    if ($this->hasEditAccess($entity)) {
      $this->_output['#attached']['library'][] = 'pagedesigner/icon';
    }
    $this->addStyles($container);
    $this->addCacheInformation($container, $entity, 'internal');
    // $this->addPatternAttachments();
    return $this;
  }

  /**
   * Render a container for a certain entity in edit mode.
   *
   * @param \Drupal\pagedesigner\Entity\Element $container
   *   The container to render.
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity the container is rendered for.
   *
   * @return \Drupal\pagedesigner\Service\Renderer
   *   The renderer service.
   */
  public function renderForEdit(Element $container, ContentEntityBase $entity = NULL) {
    if ($container == NULL) {
      return $this;
    }
    if (!$this->hasEditAccess($entity)) {
      \Drupal::messenger()->addWarning(t('You are not authorized to access this page.'));
      $this->render($container, $entity);
      return $this;
    }
    if (empty($this->_output)) {
      $this->_output = [];
    }
    if (!$this->rendering) {
      self::$styles = NULL;
    }
    if ($entity != NULL) {
      $locker = \Drupal::service('pagedesigner.locker')->setEntity($entity);
      $lock = $locker->acquire();
      if (!$lock && !$locker->hasLock()) {
        $username = 'unknown';
        $otherUser = User::load($locker->getMetaData()->owner);
        if ($otherUser != NULL) {
          $username = $otherUser->getDisplayName();
        }
        \Drupal::messenger()->addWarning(
          t('This page is currently being edited by user %user. You may access it again after the user is done editing.', ['%user' => $username])
        );
        $this->render($container, $entity, TRUE);
        return $this;
      }
    }
    $this->rendering = TRUE;
    $this->_output = $this->getHandler($container)->renderForEdit($container);
    $this->rendering = FALSE;
    $this->_output['#cache']['max-age'] = 0;
    $this->addStyles($container);
    $this->addEditorAttachments();
    $this->addPagedesigner();
    $this->addPatternAttachments();
    $this->addCacheInformation($container, $entity, 'edit');
    return $this;
  }

  /**
   * Return the markup generated by the last render.
   *
   * @return string
   *   The generated markup
   */
  public function getMarkup() {
    return Markup::create(\Drupal::service('renderer')->render($this->_output));
  }

  /**
   * Return the styles generated by the last render.
   *
   * @return string
   *   The generated styles.
   */
  public function getStyles() {
    $rendered = '';
    if (self::$styles != NULL) {
      $output = [
        'large' => '',
        'medium' => '@media (max-width: 992px)',
        'small' => '@media (max-width: 768px)',
      ];
      foreach ($output as $size => $query) {
        $styles = self::$styles->getStyles($size);
        if (!empty($styles)) {
          if (empty($query)) {
            $rendered .= $styles . "\n";
          }
          else {
            $rendered .= $query . '{' . $styles . '}' . "\n";
          }
        }
      }
    }
    return $rendered;
  }

  /**
   * Add the collected styles of the elements to the output.
   */
  protected function addStyles(Element $container) {
    if (self::$styles != NULL) {
      $rendered = $this->getStyles();
      if (!empty($rendered)) {
        $this->_output['#attached']['html_head']['pagedesigner_dynamic_css_' . $container->id()] = [
          [
            '#tag' => 'style',
            '#value' => $rendered,
            '#attributes' => [
              'id' => 'pagedesigner_dynamic_css_' . $container->id(),
              'data-pagedesigner-container' => $container->id(),
            ],
          ],
          'pagedesigner_dynamic_css_' . $container->id(),
        ];
      }
    }
  }

  /**
   * Add attachments for the WYSIWYG editor to the output.
   */
  protected function addEditorAttachments() {
    if (empty($this->_output['#attached'])) {
      $this->_output['#attached'] = [];
    }
    $config = \Drupal::config('pagedesigner.settings');
    $this->_output['#attached']['drupalSettings']['pagedesigner']['filter_format'] = $config->get('filter_format');
    $this->_output['#attached'] = array_merge(
        $this->_output['#attached'],
        \Drupal::service('plugin.manager.editor')->getAttachments([$config->get('filter_format')])
    );
  }

  /**
   * Attach pagedesigner library.
   *
   * Adds the basic pagedesigner edit and pattern libraries needed to
   * edit to the pagedesigner to the output.
   */
  protected function addPagedesigner() {
    if (empty($this->_output['#attached'])) {
      $this->_output['#attached'] = [];
    }
    $this->_output['#attached']['library'][] = 'pagedesigner/pagedesigner';
    $this->elementHandler->collectAttachments($this->_output['#attached']);
  }

  /**
   * Adds the attachments of all pattens to the output.
   */
  protected function addPatternAttachments() {
    if (empty($this->_output['#attached'])) {
      $this->_output['#attached'] = [];
    }

    $patternDefinitions = \Drupal::service('plugin.manager.ui_patterns')->getDefinitions();
    $this->elementHandler->collectPatterns($patternDefinitions);

    foreach ($patternDefinitions as $definition) {
      if ($this->elementHandler->isInTheme($definition)) {
        foreach ($definition->getLibrariesNames() as $library) {
          $this->_output['#attached']['library'][] = $library;
        }
      }
    }
  }

  /**
   * Adds the appropiate cache keys and context.
   *
   * @param \Drupal\pagedesigner\Entity\Element $container
   *   The container being rendered.
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity the output is rendered for.
   */
  protected function addCacheInformation(Element $container, ContentEntityBase $entity, $mode) {
    $build = [];
    $renderer = \Drupal::service('pagedesigner.service.renderer');
    $build['#cache'] = [
      'keys' => ['pagedesigner_content', $container->id(), $mode],
      'contexts' => ['user'],
      'max-age' => -1,
    ];
    \Drupal::service('renderer')->addCacheableDependency($build, $container);
    \Drupal::service('renderer')->addCacheableDependency($build, $entity);

    if (!empty($this->_output['#cache']) && isset($this->_output['#cache']['max-age'])) {
      $build['#cache']['max-age'] = $this->_output['#cache']['max-age'];
    }
    $this->_output = $build + $this->_output;
  }

}
